﻿<?xml version="1.0" encoding="UTF-8"?>
<!--
COPYRIGHT

Dia2CAEX - a diagram to CAEX converter
Copyright (C) 2012,2014 Stefan Theurich, Prozesskommunikation, TU Dresden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<!--
Any changes should be verified on a caex checker like http://caex.inf.tu-dresden.de
-->

<xsl:stylesheet
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:math="http://www.w3.org/2005/xpath-functions/math"
	xmlns:myfn="http://void"
	
	xmlns:uuid="java:java.util.UUID"
	
	xmlns:dia="http://www.lysator.liu.se/~alla/dia/"

	exclude-result-prefixes="xsl xsi fn math myfn uuid dia"
	
	version="2.0">
	
	<xsl:import href="Dia2CAEX_importerror.xsl"/>

	<!-- TODO other objects -->
	<xsl:import href="DiaIEC62424.xsl"/>
	<xsl:import href="DiaChemEng.xsl"/>

	<xsl:output method="xml" indent="yes"  />
	<xsl:strip-space elements="*"/>
	
	<xsl:function name="myfn:substring-after-last" as="xs:string">
		<xsl:param name="input" as="xs:string" />
		<xsl:param name="substr" as="xs:string" />
		<xsl:value-of select="if ($substr) then if (fn:contains($input, $substr)) then tokenize ($input, $substr)[last()] else $input else $input" />
	</xsl:function>
	
	
	<xsl:template name="main" match="/">
		<CAEXFile
			SchemaVersion="2.15"
			xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
			xmlns:xs="http://www.w3.org/2001/XMLSchema"
			xmlns:pix="http://www.plt.rwth-aachen.de/perma/PandIX_FP">
			<xsl:attribute name="xsi:noNamespaceSchemaLocation">CAEX_ClassModel_V2.15.xsd</xsl:attribute>
			<xsl:attribute name="FileName">
				<xsl:value-of select="fn:replace(myfn:substring-after-last(base-uri(), '/'), '.dia', '.aml')" />
			</xsl:attribute>
			<AdditionalInformation AutomationMLVersion="2.0" />
			<xsl:apply-templates select="dia:diagram/dia:layer/dia:object[@type='IEC62424 - externalreference']" mode="extref" />
			<!-- 
			TODO namen glatt ziehen und wieder das generische!
		   <ExternalReference Path="PandIX_ICL_V600.aml" Alias="PandIX_ICL"/>
		   <ExternalReference Path="PandIX_PCE_V600.aml" Alias="PandIX_PCE"/>
		   <ExternalReference Path="PandIX_PPE-BasicLib_V600.aml" Alias="PandIX_PPE"/>
		   <ExternalReference Path="stth_PPE-BasicLib_V600.aml" Alias="stth_PPE"/>
		   <ExternalReference Path="stth_PPE-SUC-ExampleLib_V600.aml" Alias="stth_SUC"/>
			 -->

			<InstanceHierarchy Name="Dia-Export">
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<xsl:apply-templates select="dia:diagram/dia:layer/dia:group" />
			</InstanceHierarchy>
		</CAEXFile>
	</xsl:template>
	
	<xsl:template name="extref" match="dia:object" mode="extref">
		<xsl:choose>
			<xsl:when test="@type='IEC62424 - externalreference'">
				<ExternalReference>
				<xsl:attribute name="Path">
					<xsl:value-of select="replace(dia:attribute[@name='custom:Filename']/dia:string,'#', '')" />
				</xsl:attribute>
				<xsl:attribute name="Alias">
					<xsl:value-of select="replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', '')" />
				</xsl:attribute>
				</ExternalReference>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message terminate="yes">
					<xsl:text>error in extref: type != 'IEC62424 - externalreference' at diagram position </xsl:text>
					<xsl:apply-templates select="." mode="objectpos" />
				</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="extint" match="dia:object[@type='IEC62424 - externalinterface']" mode="extint">
		<ExternalInterface>
			<xsl:attribute name="RefBaseClassPath">
				<xsl:value-of select="fn:replace(dia:attribute[@name='custom:RefBaseClassPath']/dia:string,'#', '')" />
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="extifname" />
			</xsl:attribute>
			<!-- TODO other interfaces -->
			<xsl:choose>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/ProductConnectionPoint#'">
					<Attribute Name="DN">
						<Value>32</Value>
					</Attribute>
					<Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/MeasurementPoint#'">
					<Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/SensorProcessInterface#'">
					<!-- <Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>  -->
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/SignalSource#'">
					<!-- <Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>  -->
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/ProcessConnectionInterface#'">
					<!-- <Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>  -->
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/ActuationInputPoint#'">
					<!-- <Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>  -->
				</xsl:when>
				<xsl:when test="dia:attribute[@name='custom:RefBaseClassPath']/dia:string='#PandIX_ICL@PandIX_ICL/ProductEnvironmentPoint#'">
					<Attribute Name="DN">
						<Value>32</Value>
					</Attribute>
					<Attribute Name="Position">
						<Value>S01P0:</Value>
					</Attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:message terminate="no">
						<xsl:text>error in extint: unknown interface </xsl:text>
						<xsl:value-of select="dia:attribute[@name='custom:RefBaseClassPath']/dia:string" />
						<xsl:text> at diagram position </xsl:text>
						<xsl:apply-templates select="." mode="objectpos" />
						<xsl:text>. Change in properties Dialog in Dia, see PandIX_ICL.aml.</xsl:text>
					</xsl:message>
				</xsl:otherwise>
			</xsl:choose>
		</ExternalInterface>
	</xsl:template>
	
	<!-- 
		<dia:attribute name="poly_points">
			<dia:point val="13.0534,12.036"/>
			<dia:point val="13,10"/>
			<dia:point val="28,10"/>
			<dia:point val="28.0534,12.011"/>
		</dia:attribute>
	 -->
	<xsl:template name="pipeform">
		<xsl:param name="polypoints" />
		<xsl:param name="index" />
		<xsl:param name="count" />
		<xsl:variable name="p1" select="$polypoints/dia:point[$index - 1]/@val" />
		<xsl:variable name="p2" select="$polypoints/dia:point[$index]/@val" />
		<xsl:variable name="x1" select="fn:number(substring-before($p1, ','))" />
		<xsl:variable name="y1" select="fn:number(substring-after($p1, ','))" />
		<xsl:variable name="x2" select="fn:number(substring-before($p2, ','))" />
		<xsl:variable name="y2" select="fn:number(substring-after($p2, ','))" />
		<xsl:if test="$index &gt; 2">
			<xsl:text>;</xsl:text>
		</xsl:if>
		<xsl:text>(</xsl:text>
		<!-- TODO instead of xsl:choose <xsl:value-of select="math:sqrt(math:sqr($x2 - $x1) + math:sqr($y2 - $y1))" /> -->
		<xsl:choose>
			<xsl:when test="fn:abs($x2 - $x1) &gt; fn:abs($y2 - $y1)">
				<xsl:value-of select="fn:abs($x2 - $x1)" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="fn:abs($y2 - $y1)" />
			</xsl:otherwise>
		</xsl:choose>
		<xsl:text>;</xsl:text>
		<xsl:value-of select="$y2 - $y1" />
		<xsl:text>)</xsl:text>
		<!-- 
		<xsl:value-of select="$p1" />__p1
		<xsl:value-of select="$p2" />__p2
		<xsl:value-of select="$x1" />__x1
		<xsl:value-of select="$x2" />__x2
		<xsl:value-of select="$y1" />__y1
		<xsl:value-of select="$y2" />__y2
		 -->
		<xsl:if test="$index &lt; $count">
			<xsl:call-template name="pipeform">
				<xsl:with-param name="polypoints" select="$polypoints" />
				<xsl:with-param name="index" select="$index + 1" />
				<xsl:with-param name="count" select="$count" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="intelem" match="dia:object" mode="intelem">
		<xsl:choose>
			<xsl:when test="@type='Standard - PolyLine'">
				<xsl:variable name="lasthandle" select="dia:connections/dia:connection[last()]/@handle" />
				<xsl:if test="dia:connections/dia:connection[1]/@handle!='0'">
					<xsl:message terminate="no">
						<xsl:text>polyline start is not connected at diagram position </xsl:text>
						<xsl:apply-templates select="." mode="handlepos">
							<xsl:with-param name="handle" select="'0'" />
						</xsl:apply-templates>
					</xsl:message>
				</xsl:if>
				<xsl:if test="count(dia:attribute[@name='poly_points']/dia:point)-1!=$lasthandle">
					<xsl:message terminate="no">
						<xsl:text>polyline end is not connected at diagram position </xsl:text>
						<xsl:apply-templates select="." mode="handlepos">
							<xsl:with-param name="handle" select="$lasthandle" />
						</xsl:apply-templates>
					</xsl:message>
				</xsl:if>
				<InternalElement>
					<xsl:attribute name="Name">
						<xsl:apply-templates select="." mode="intelemname" />
					</xsl:attribute>
					<xsl:attribute name="ID">
						<xsl:value-of select="uuid:randomUUID()" />
					</xsl:attribute>
					<xsl:choose>
						<xsl:when test="dia:attribute[@name='line_style']/dia:enum[@val='1']">
							<Attribute Name="ISO10628Type">
								<Value>C0109</Value>
							</Attribute>			
							<ExternalInterface Name="Start" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource">
								<!-- <Attribute Name="Position">
									<Value>S01P0:</Value>
								</Attribute> -->
							</ExternalInterface>
							<ExternalInterface Name="End" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink">
								<!-- <Attribute Name="Position">
									<Value>S01P0:</Value>
								</Attribute> -->
							</ExternalInterface>
							<RoleRequirements RefBaseRoleClassPath="stth_PPE@stth_PPE_Lib/SignalLine" />
						</xsl:when>
						<!-- TODO other connections -->
						<xsl:otherwise>
							<!-- 
							<Attribute Name="NominalVolume" Unit="m^3" AttributeDataType="xs:double">
								<Value>0.0</Value>
							</Attribute>
							 -->
							<Attribute Name="PN">
								<Value>20</Value>
							</Attribute>
							<Attribute Name="TotalVolume">
								<Value>0.0</Value>
							</Attribute>
							<Attribute Name="Position">
								<Value>S01P0:</Value>
							</Attribute>
							<Attribute Name="Form">
							<!-- 
								<dia:attribute name="poly_points">
									<dia:point val="13.0534,12.036"/>
									<dia:point val="13,10"/>
									<dia:point val="28,10"/>
									<dia:point val="28.0534,12.011"/>
								</dia:attribute>
							 -->
								<Value>
									<xsl:text>S01F1:</xsl:text>
									<xsl:call-template name="pipeform">
										<xsl:with-param name="polypoints" select="dia:attribute[@name='poly_points']" />
										<xsl:with-param name="index" select="2" />
										<xsl:with-param name="count" select="count(dia:attribute[@name='poly_points']/dia:point)" />
									</xsl:call-template>
								</Value>
							</Attribute>
							<Attribute Name="ISO10628Type">
								<Value>X8155</Value>
							</Attribute>			
							<Attribute Name="DN">
								<Value>32</Value>
							</Attribute>			
							<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
								<Attribute Name="DN">
									<Value>32</Value>
								</Attribute>
								<Attribute Name="Position">
									<Value>S01P0:</Value>
								</Attribute>
							</ExternalInterface>
							<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
								<Attribute Name="DN">
									<Value>32</Value>
								</Attribute>
								<Attribute Name="Position">
									<Value>S01P0:</Value>
								</Attribute>
							</ExternalInterface>
							<xsl:for-each select="dia:connections/dia:connection">
								<xsl:if test="@handle!='0' and @handle!=$lasthandle">
								<ExternalInterface RefBaseClassPath="PandIX_ICL@PandIX_ICL/MeasurementPoint">
									<xsl:attribute name="Name">
										<xsl:text>M</xsl:text>
										<xsl:value-of select="@handle" />
									</xsl:attribute>
									<Attribute Name="Position">
										<Value>S01P0:</Value>
									</Attribute>
									</ExternalInterface>
								</xsl:if>
							</xsl:for-each>
							<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PipeRequest" />
						</xsl:otherwise>
					</xsl:choose>
				</InternalElement>
			</xsl:when>
			<xsl:when test="(@type='Standard - Line') or (@type='Standard - ZigZagLine')">
				<xsl:variable name="start" select="dia:connections/dia:connection[1]" />
				<xsl:variable name="end" select="dia:connections/dia:connection[2]" />
				<xsl:choose>
					<xsl:when test="$start and $end">
						<xsl:variable name="start_type" select="//dia:object[@id=$start/@to]/@type" />
						<xsl:choose>
							<xsl:when test="($start_type='IEC62424 - measurement') or ($start_type='IEC62424 - measurement_diff')" >
								<InternalElement>
									<xsl:attribute name="Name">
										<xsl:apply-templates select="." mode="intelemname" />
									</xsl:attribute>
									<xsl:attribute name="ID">
										<xsl:value-of select="uuid:randomUUID()" />
									</xsl:attribute>
									<ExternalInterface Name="M" RefBaseClassPath="PandIX_ICL@PandIX_ICL/MeasurementPoint">
										<Attribute Name="Position">
											<Value>S01P0:</Value>
										</Attribute>
									</ExternalInterface>
									<ExternalInterface Name="P" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
										<Attribute Name="Position">
											<Value>S01P0:</Value>
										</Attribute>
										<Attribute Name="DN">
											<Value>32</Value>
										</Attribute>
									</ExternalInterface>
								</InternalElement>
							</xsl:when>
							<xsl:when test="$start_type='IEC62424 - actuator'" />
							<!-- nothing to do -->
							<xsl:otherwise>
								<xsl:message terminate="no">
									<xsl:text>Standard line id= </xsl:text>
									<xsl:value-of select="@id" />
									<xsl:text> start handle is not connected to a measurement or actuator. Only use standard lines to connect measurements/actuators ProcessPoints to equipments ProductConnectionPoints/ActuationInputPoints! See diagram position at diagram position </xsl:text>
									<xsl:apply-templates select="." mode="handlepos">
										<xsl:with-param name="handle" select="'0'" />
									</xsl:apply-templates>
								</xsl:message>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
						<xsl:if test="not($start) or $start/@handle!='0'">
							<xsl:message terminate="no">
								<xsl:text>line start is not connected at diagram position </xsl:text>
								<xsl:apply-templates select="." mode="handlepos">
									<xsl:with-param name="handle" select="'0'" />
								</xsl:apply-templates>
							</xsl:message>
						</xsl:if>
						<xsl:if test="not($end)">
							<xsl:message terminate="no">
								<xsl:text>line end is not connected at diagram position </xsl:text>
								<xsl:apply-templates select="." mode="handlepos">
									<xsl:with-param name="handle" select="'1'" />
								</xsl:apply-templates>
							</xsl:message>
						</xsl:if>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-imports/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="group" match="dia:group">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<xsl:apply-templates select="dia:object[@type='IEC62424 - externalinterface']"  mode="extint"/>
			<xsl:apply-templates select="dia:object[@type!='IEC62424 - externalinterface']"  mode="intelem"/>
			<xsl:apply-templates select="dia:group" />
			<xsl:apply-templates select="dia:object/dia:connections" />
		</InternalElement>
	</xsl:template>

	<xsl:function name="myfn:correctNameForLink" as="xs:string">
		<xsl:param name="name" as="xs:string"/>
		<xsl:variable name="escape" select="fn:replace(fn:replace($name, '\[', '&amp;#91;'), '\]', '&amp;#93;')" />
		<xsl:choose>
			<xsl:when test="fn:translate($name, '@/:.', '') = $name">
				<xsl:value-of select="$escape" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="fn:concat('[', $escape, ']')" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:function>

	<xsl:template name="intlink" match="dia:connections">
		<xsl:choose>
			<xsl:when test="../@type='Standard - PolyLine'">
				<xsl:for-each select="dia:connection">
					<InternalLink>
						<xsl:variable name="targetid" select="@to" />
						<xsl:variable name="connectorparentgroup">
							<xsl:apply-templates select="../../.." mode="intelemname" />
						</xsl:variable>
						<xsl:variable name="targetparentgroup0">
							<xsl:apply-templates select="//dia:object[@id=$targetid]/.." mode="intelemname" />
						</xsl:variable>
						<xsl:variable name="targetparentgroup1">
							<xsl:apply-templates select="//dia:object[@id=$targetid]/../.." mode="intelemname" />
						</xsl:variable>
						<xsl:variable name="targetparentgroup2">
							<xsl:apply-templates select="//dia:object[@id=$targetid]/../../.." mode="intelemname" />
						</xsl:variable>
						<xsl:variable name="targetparentgroup3">
							<xsl:apply-templates select="//dia:object[@id=$targetid]/../../../.." mode="intelemname" />
						</xsl:variable>
						<xsl:variable name="intelemname">
							<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="intelemname" />
						</xsl:variable>
										
						<xsl:attribute name="Name">
							<xsl:text>link_</xsl:text>
							<xsl:value-of select="$intelemname" />
							<xsl:text>_</xsl:text>
							<xsl:apply-templates select="../.." mode="intelemname" />
						</xsl:attribute>
						
						<!-- Group hierarchy & Interface -->
						<xsl:attribute name="RefPartnerSideA">
							<xsl:choose>
								<xsl:when test="$connectorparentgroup = $targetparentgroup0">
									<xsl:choose>
										<!-- connector to explicit external interface of the group (A) -->
										<xsl:when test="//dia:object[@id=$targetid]/@type='IEC62424 - externalinterface'">
											<xsl:if test="@connection!='0'">
												<xsl:message terminate="no">
													<xsl:text>error in intlink: inner connection connected to outside connector at diagram position </xsl:text>
													<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="objectpos" />
												</xsl:message>
											</xsl:if> 
										</xsl:when>
										<!-- connector to an object of the group (B) -->
										<xsl:otherwise>
											<xsl:value-of select="myfn:correctNameForLink($intelemname)" />
											<xsl:text>:</xsl:text>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<xsl:when test="$connectorparentgroup=$targetparentgroup1">
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup0)" />
									<xsl:choose>
										<!-- connector to explicit external interface of the nested group (C) -->
										<xsl:when test="//dia:object[@id=$targetid]/@type='IEC62424 - externalinterface'">
											<xsl:if test="@connection!='1'">
												<xsl:message terminate="no">
													<xsl:text>error in intlink: outer connection connected to inside connector at diagram position </xsl:text>
													<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="objectpos" />
												</xsl:message>
											</xsl:if>
										</xsl:when>
										<!-- connector to an object of the nested group (D) -->
										<xsl:otherwise>
											<xsl:text>/</xsl:text>
											<xsl:value-of select="myfn:correctNameForLink($intelemname)" />
										</xsl:otherwise>
									</xsl:choose>
									<xsl:text>:</xsl:text>
								</xsl:when>
								<xsl:when test="$connectorparentgroup=$targetparentgroup2">
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup1)" />
									<xsl:text>/</xsl:text>
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup0)" />
									<xsl:choose>
										<!-- connector to explicit external interface of the nested group (C) -->
										<xsl:when test="//dia:object[@id=$targetid]/@type='IEC62424 - externalinterface'">
											<xsl:if test="@connection!='1'">
												<xsl:message terminate="no">
													<xsl:text>error in intlink: outer connection connected to inside connector at diagram position </xsl:text>
													<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="objectpos" />
												</xsl:message>
											</xsl:if>
										</xsl:when>
										<!-- connector to an object of the nested group (D) -->
										<xsl:otherwise>
											<xsl:text>/</xsl:text>
											<xsl:value-of select="myfn:correctNameForLink($intelemname)" />
										</xsl:otherwise>
									</xsl:choose>
									<xsl:text>:</xsl:text>
								</xsl:when>
								<xsl:when test="$connectorparentgroup=$targetparentgroup3">
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup2)" />
									<xsl:text>/</xsl:text>
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup1)" />
									<xsl:text>/</xsl:text>
									<xsl:value-of select="myfn:correctNameForLink($targetparentgroup0)" />
									<xsl:choose>
										<!-- connector to explicit external interface of the nested group (C) -->
										<xsl:when test="//dia:object[@id=$targetid]/@type='IEC62424 - externalinterface'">
											<xsl:if test="@connection!='1'">
												<xsl:message terminate="no">
													<xsl:text>error in intlink: outer connection connected to inside connector at diagram position </xsl:text>
													<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="objectpos" />
												</xsl:message>
											</xsl:if>
										</xsl:when>
										<!-- connector to an object of the nested group (D) -->
										<xsl:otherwise>
											<xsl:text>/</xsl:text>
											<xsl:value-of select="myfn:correctNameForLink($intelemname)" />
										</xsl:otherwise>
									</xsl:choose>
									<xsl:text>:</xsl:text>
								</xsl:when>
								<xsl:otherwise>
									<xsl:message terminate="yes">
										<xsl:text>error in intlink: could not track a connection of line _near_ diagram position </xsl:text>
									    <xsl:apply-templates select=".." mode="objectpos" />
									</xsl:message>
								</xsl:otherwise>
							</xsl:choose>
						
							<xsl:variable name="tmp">
								<xsl:apply-templates select="//dia:object[@id=$targetid]" mode="extifname">
									<xsl:with-param name="connection">
										<xsl:value-of select="@connection" />
									</xsl:with-param>
								</xsl:apply-templates>
							</xsl:variable>
							<xsl:value-of select="myfn:correctNameForLink($tmp)" />
						</xsl:attribute>
						
						<!-- Group-self:ConnectionId -->
						<xsl:attribute name="RefPartnerSideB">
							<xsl:variable name="tmp">
								<xsl:apply-templates select="../.." mode="intelemname" />
							</xsl:variable>
							<xsl:value-of select="myfn:correctNameForLink($tmp)" />
							
							<xsl:text>:</xsl:text>
							<!-- do not use [, ], @, /, :, . in names! -->
							<xsl:variable name="endhandle" select="count(../../dia:attribute[@name='poly_points']/dia:point)-1" />
							<xsl:choose>
								<xsl:when test="../../dia:attribute[@name='line_style']/dia:enum[@val='1']">
									<!-- SignalLine -->
									<xsl:choose>
										<xsl:when test="@handle='0'">
											<xsl:text>Start</xsl:text>
										</xsl:when>
										<xsl:when test="@handle=$endhandle">
											<xsl:text>End</xsl:text>
										</xsl:when>
										<xsl:otherwise>
											<xsl:message>
												<xsl:text>error in intlink: middle connector of signal line is connected at diagram position </xsl:text>
												<xsl:apply-templates select="../.." mode="handlepos" />
											</xsl:message>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:when>
								<!-- TODO other connectors-->
								<xsl:otherwise>
									<xsl:choose>
										<!-- pipe -->
										<xsl:when test="@handle='0'">
											<xsl:text>PIn</xsl:text>
										</xsl:when>
										<xsl:when test="@handle=$endhandle">
											<xsl:text>POut</xsl:text>
										</xsl:when>
										<xsl:otherwise>
											<xsl:text>M</xsl:text>
											<xsl:value-of select="@handle" />
										</xsl:otherwise>
									</xsl:choose>
								</xsl:otherwise>
							</xsl:choose>
						</xsl:attribute>
					</InternalLink>
				</xsl:for-each>
			</xsl:when>
			<xsl:when test="(../@type='Standard - Line') or (../@type='Standard - ZigZagLine')">
 				<xsl:variable name="start" select="dia:connection[1]" />
				<xsl:variable name="end" select="dia:connection[2]" />
				<xsl:choose>
					<xsl:when test="$start and $end">
						<xsl:variable name="start_obj" select="//dia:object[@id=$start/@to]" />
						<xsl:variable name="end_obj" select="//dia:object[@id=$end/@to]" />
						<xsl:choose>
							<xsl:when test="($start_obj/@type='IEC62424 - measurement') or ($start_obj/@type='IEC62424 - measurement_diff')" >
								<InternalLink>
									<xsl:attribute name="Name">
										<xsl:text>link_</xsl:text>
										<xsl:apply-templates select="$start_obj" mode="intelemname" />
										<xsl:text>_</xsl:text>
										<xsl:apply-templates select=".." mode="intelemname" />
									</xsl:attribute>
					
									<xsl:attribute name="RefPartnerSideA">
										<xsl:variable name="tmp1">
											<xsl:apply-templates select="$start_obj" mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp1)" />
										
										<xsl:text>:</xsl:text>
										
										<xsl:variable name="tmp2">
											<xsl:apply-templates select="$start_obj" mode="extifname">
												<xsl:with-param name="connection">
													<xsl:value-of select="$start/@connection" />
												</xsl:with-param>
											</xsl:apply-templates>
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp2)" />
									</xsl:attribute>
									
									<xsl:attribute name="RefPartnerSideB">
										<xsl:variable name="tmp">
											<xsl:apply-templates select=".." mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp)" />
										
										<xsl:text>:M</xsl:text>
									</xsl:attribute>
								</InternalLink>
								<InternalLink>
									<xsl:attribute name="Name">
										<xsl:text>link_</xsl:text>
										<xsl:apply-templates select="$end_obj" mode="intelemname" />
										<xsl:text>_</xsl:text>
										<xsl:apply-templates select=".." mode="intelemname" />
									</xsl:attribute>
					
									<xsl:attribute name="RefPartnerSideA">
										<xsl:variable name="tmp1">
											<xsl:apply-templates select="$end_obj" mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp1)" />

										<xsl:text>:</xsl:text>

										<xsl:variable name="tmp2">
											<xsl:apply-templates select="$end_obj" mode="extifname">
												<xsl:with-param name="connection">
													<xsl:value-of select="$end/@connection" />
												</xsl:with-param>
											</xsl:apply-templates>
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp2)" />
									</xsl:attribute>
									
									<xsl:attribute name="RefPartnerSideB">
										<xsl:variable name="tmp">
											<xsl:apply-templates select=".." mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp)" />
										
										<xsl:text>:P</xsl:text>
									</xsl:attribute>
								</InternalLink>
							</xsl:when>
							<xsl:when test="$start_obj/@type='IEC62424 - actuator'" >
								<InternalLink>
									<xsl:attribute name="Name">
										<xsl:text>link_</xsl:text>
										<xsl:apply-templates select="$start_obj" mode="intelemname" />
										<xsl:text>_</xsl:text>
										<xsl:apply-templates select="$end_obj" mode="intelemname" />
									</xsl:attribute>
					
									<xsl:attribute name="RefPartnerSideA">
										<xsl:variable name="tmp1">
											<xsl:apply-templates select="$start_obj" mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp1)" />

										<xsl:text>:</xsl:text>
										
										<xsl:variable name="tmp2">
											<xsl:apply-templates select="$start_obj" mode="extifname">
												<xsl:with-param name="connection">
													<xsl:value-of select="$start/@connection" />
												</xsl:with-param>
											</xsl:apply-templates>
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp2)" />
									</xsl:attribute>
									
									<xsl:attribute name="RefPartnerSideB">
										<xsl:variable name="tmp1">
											<xsl:apply-templates select="$end_obj" mode="intelemname" />
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp1)" />

										<xsl:text>:</xsl:text>

										<xsl:variable name="tmp2">
											<xsl:apply-templates select="$end_obj" mode="extifname">
												<xsl:with-param name="connection">
													<xsl:value-of select="$end/@connection" />
												</xsl:with-param>
											</xsl:apply-templates>
										</xsl:variable>
										<xsl:value-of select="myfn:correctNameForLink($tmp2)" />
									</xsl:attribute>
								</InternalLink>
							</xsl:when>
						</xsl:choose>
					</xsl:when>
					<xsl:otherwise>
					</xsl:otherwise>
				</xsl:choose>
 			</xsl:when>
			<xsl:otherwise>
				<xsl:message terminate="yes">
					<xsl:text>unsupported dia object of type </xsl:text>
					<xsl:value-of select="../@type" />
					<xsl:text> has connections at diagram position </xsl:text>
					<xsl:apply-templates select=".." mode="objectpos" />
				</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="intelemname" match="dia:object" mode="intelemname">
		<xsl:variable name="text" select="dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']" />
		<xsl:choose>
			<xsl:when test="@type='IEC62424 - externalinterface'">
				<xsl:apply-templates select="." mode="extifname" />
			</xsl:when>
			<xsl:when test="@type='Standard - PolyLine'">
				<xsl:choose>
					<xsl:when test="dia:attribute[@name='line_style']/dia:enum[@val='1']">
						<xsl:text>SignalLine</xsl:text>
						<xsl:value-of select="@id" />
					</xsl:when>
					<!-- TODO etc. -->
					<xsl:otherwise>
						<xsl:text>Pipe</xsl:text>
						<xsl:value-of select="@id" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="(@type='Standard - Line') or (@type='Standard - ZigZagLine')">
				<xsl:variable name="myid" select="@id" />
				<xsl:variable name="start" select="dia:connections/dia:connection[1]" />
				<xsl:variable name="end" select="dia:connections/dia:connection[2]" />
				<xsl:choose>
					<xsl:when test="$start and $end">
						<xsl:variable name="start_obj" select="//dia:object[@id=$start/@to]" />
						<xsl:choose>
							<xsl:when test="$start_obj/@type='IEC62424 - measurement'" >
								<xsl:text>MeasurementMount</xsl:text>
								<xsl:value-of select="@id" />
							</xsl:when>
							<xsl:when test="$start_obj/@type='IEC62424 - measurement_diff'" >
								<xsl:text>MeasurementMount</xsl:text>
								<xsl:value-of select="@id" />
								<xsl:choose>
									<xsl:when test="$start/@connection=1" >
										<xsl:text>1</xsl:text>
									</xsl:when>
									<xsl:when test="$start/@connection=0">
										<xsl:text>2</xsl:text>
									</xsl:when>
									<xsl:otherwise>
										<xsl:message>
											<xsl:text>error in intelemname: zigzagline is connected to wrong handle at measurement</xsl:text>
										</xsl:message>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:when>
							<xsl:otherwise>
								<xsl:message>
									<xsl:text>error in intelemname: name requested for single line / zigzagline which is not connected to a measurement and equipment</xsl:text>
								</xsl:message>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:when>
				</xsl:choose>
			</xsl:when>
			<!-- TO-DO is this a good idea? this disables all forced overrides.  -->
			<xsl:when test="$text">
				<xsl:variable name="trimmedtext"  select="replace(replace(replace($text/dia:string,'#', ''),'&#xA;',''),' ','')" />
				<xsl:choose>
					<xsl:when test="$trimmedtext">
						<xsl:value-of select="$trimmedtext" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:message terminate="no">
							<xsl:text>error in intelemname: name of the object of type </xsl:text>
							<xsl:value-of select="@type" />
							<xsl:text> at diagram position </xsl:text>
							<xsl:apply-templates select="." mode="objectpos" />
							<xsl:text> is empty</xsl:text>
						</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:otherwise>
				<xsl:variable name="result">
					<xsl:apply-imports />
				</xsl:variable>
				<xsl:choose>
					<xsl:when test="$result">
						<xsl:value-of select="$result" />
					</xsl:when>
					<xsl:otherwise>
						<xsl:message terminate="yes">
							<xsl:text>error in intelemname: no name for the internal element of type </xsl:text>
							<xsl:value-of select="@type" />
							<xsl:text> could be found</xsl:text>
						</xsl:message>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>	
	
	<xsl:template name="intelemname_g" match="dia:group" mode="intelemname">
		<xsl:variable name="groupnames" select="dia:object[@type='IEC62424 - internal_element_name']" />
		<xsl:choose>
			<xsl:when test="$groupnames">
				<xsl:if test="$groupnames[2]">
					<xsl:message>
						<xsl:text>there are more than one objects of type internal_element_name in a group at diagram positions </xsl:text>
						<xsl:apply-templates select="$groupnames" mode="objectpos" />
						<xsl:text> and </xsl:text>
						<xsl:apply-templates select="$groupnames[2]" mode="objectpos" />
					</xsl:message>
				</xsl:if>
				<xsl:apply-templates select="$groupnames" mode="intelemname" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:text>IntElemWith</xsl:text>
				<xsl:value-of select="dia:object[1]/@id" />
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>	

	<xsl:template name="extifname" match="dia:object" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="@type='IEC62424 - externalinterface'">
				<xsl:value-of select="fn:replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', '')" />
			</xsl:when>
			<xsl:when test="@type='Standard - PolyLine'">
				<xsl:variable name="endhandle" select="count(../../dia:attribute[@name='poly_points']/dia:point)-1" />
				<xsl:choose>
					<!-- TO-DO Signal line etc. -->
					<!-- TODO extifname? -->
					<xsl:when test="@handle='0'">
						<xsl:text>PIn</xsl:text>
					</xsl:when>
					<xsl:when test="@handle=$endhandle">
						<xsl:text>POut</xsl:text>
					</xsl:when>
					<xsl:otherwise>>
						<xsl:text>M</xsl:text>
						<xsl:value-of select="@handle" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="@type='Standard - Line'">
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-imports>
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-imports>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	<xsl:template name="objectpos" match="dia:object" mode="objectpos">
		<xsl:text>(</xsl:text>
		<xsl:value-of select="dia:attribute[@name='obj_pos']/dia:point/@val" />
		<xsl:text>)</xsl:text>
	</xsl:template>
	
	<xsl:template name="handlepospolyline" match="dia:object[@type='Standard - PolyLine']" mode="handlepos">
		<xsl:param name="handle" />
		<xsl:text>(</xsl:text>
		<xsl:value-of select="dia:attribute[@name='poly_points']/dia:point[number($handle)+1]/@val" />
		<xsl:text>)</xsl:text>
	</xsl:template>
		
	<xsl:template name="handleposline" match="dia:object[@type='Standard - Line']" mode="handlepos">
		<xsl:param name="handle" />
		<xsl:text>(</xsl:text>
		<xsl:value-of select="dia:attribute[@name='conn_endpoints']/dia:point[number($handle)+1]/@val" />
		<xsl:text>)</xsl:text>
	</xsl:template>

	<xsl:template name="unsupported_connector" match="dia:object" mode="unsupported_connector">
		<xsl:param name="connection" />
		<xsl:variable name="myid" select="@id" />
		<xsl:variable name="connobj" select="//dia:connection[@to=$myid and @connection=$connection]" />
		<xsl:message terminate="no">
			<xsl:text>error in extifname: connected connectionpoint </xsl:text>
			<xsl:value-of select="$connection"></xsl:value-of>
			<xsl:text> of </xsl:text>
			<xsl:value-of select="@type"></xsl:value-of>
			<xsl:text> (id=</xsl:text>
			<xsl:value-of select="@id"></xsl:value-of>
			<xsl:text>) at diagram position (object) </xsl:text>
			<xsl:apply-templates select="." mode="objectpos" />
			<xsl:text> shall not be used; connector pos: </xsl:text>
			<xsl:apply-templates select="$connobj/../.." mode="handlepos">
				<xsl:with-param name="handle" select="$connobj/@handle" />
			</xsl:apply-templates>
		</xsl:message>
	</xsl:template>
	
	<xsl:template name="require_connected_connector" match="dia:object" mode="require_connected_connector">
		<xsl:param name="connection" />
		<xsl:variable name="myid" select="@id" />
		<xsl:variable name="myconn" select="//dia:object[@type='Standard - PolyLine' or @type='Standard - Line' or @type='Standard - ZigZagLine']/dia:connections/dia:connection[@to=$myid and @connection=$connection]" />
		<xsl:if test="not($myconn)">
			<xsl:message terminate="no">
				<xsl:text>connector requires a connected object at connection </xsl:text>
				<xsl:apply-templates select="." mode="extifname">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
				<xsl:text> (</xsl:text>
				<xsl:value-of select="$connection" />
				<xsl:text>) of object </xsl:text>
				<xsl:apply-templates select="." mode="intelemname" />
				 <xsl:text> at diagram position (object position) </xsl:text>
				<xsl:apply-templates select="." mode="objectpos" />
			</xsl:message>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="require_outgoing_connection" match="dia:object" mode="require_outgoing_connection">
		<xsl:param name="connection" />
		<xsl:variable name="myid" select="@id" />
		<xsl:variable name="myOutconn" select="//dia:object[@type='Standard - PolyLine' or @type='Standard - Line' or @type='Standard - ZigZagLine']/dia:connections/dia:connection[@to=$myid and @connection=$connection]" />
		<xsl:if test="$myOutconn and $myOutconn/@handle!='0'" >
			<xsl:message terminate="no">
				<xsl:text>line points towards output-only-connector </xsl:text>
				<xsl:apply-templates select="." mode="extifname">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
				<xsl:text> (</xsl:text>
				<xsl:value-of select="$connection" />
				<xsl:text>) of object </xsl:text>
				<xsl:apply-templates select="." mode="intelemname" />
				 <xsl:text> at diagram position (connector position) </xsl:text>
				<xsl:apply-templates select="$myOutconn/../.." mode="handlepos">
					<xsl:with-param name="handle" select="$myOutconn/@handle" />
				</xsl:apply-templates>
			</xsl:message>
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="require_incoming_connection" match="dia:object" mode="require_incoming_connection">
		<xsl:param name="connection" />
		<xsl:variable name="myid" select="@id" />
		<xsl:variable name="myInconn" select="//dia:object[@type='Standard - PolyLine' or @type='Standard - Line' or @type='Standard - ZigZagLine']/dia:connections/dia:connection[@to=$myid and @connection=$connection]" />
		<xsl:if test="$myInconn and $myInconn/@handle='0'" >
			<xsl:message terminate="no">
				<xsl:text>line points towards input-only-connector </xsl:text>
				<xsl:apply-templates select="." mode="extifname">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
				<xsl:text> (</xsl:text>
				<xsl:value-of select="$connection" />
				<xsl:text>) of object </xsl:text>
				<xsl:apply-templates select="." mode="intelemname" />
				 <xsl:text> at diagram position (connector position) </xsl:text>
				<xsl:apply-templates select="$myInconn/../.." mode="handlepos">
					<xsl:with-param name="handle" select="$myInconn/@handle" />
				</xsl:apply-templates>
			</xsl:message>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
