<?xml version="1.0" encoding="UTF-8"?>
<!--
COPYRIGHT

Dia2CAEX - a diagram to CAEX converter
Copyright (C) 2012,2014 Stefan Theurich, Prozesskommunikation, TU Dresden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<!--
Any changes should be verified on a caex checker like http://caex.inf.tu-dresden.de
-->

<xsl:stylesheet
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		
		xmlns:fn="http://www.w3.org/2005/xpath-functions"
		xmlns:uuid="java:java.util.UUID"
		
		xmlns:dia="http://www.lysator.liu.se/~alla/dia/"
		
		exclude-result-prefixes="xsl xsi fn uuid dia"
		
		version="2.0">

	<!-- every dia figure needs two or three templates:
		
		mandatory: <xsl:template match="dia:object[@type='???']" mode="intelem" />
		prints out the <InternalElement /> tag that declares the object
		
		optional: 
		<xsl:template match="dia:object[@type='???']" mode="intelemname" />
		prints out the plain name of th element if there is a textbox in the shape,
		this text (without any line breaks and spaces) will be used 
		as name and this template will only be called, if there is no string given
		
		mandatory <xsl:template match="dia:object[@type='???']" mode="extifname" /> <xsl:param name="connection" />
		prints the name of  the external interface of the object
		corresponding to the given connector number (as string)
	-->

	<xsl:template match="dia:object[@type='chemeng - fan']" mode="intelem">
		<!-- Fan or Stirrer -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - fan']" mode="intelemname">
		<!-- Fan or Stirrer -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - fan']" mode="extifname">
		<!-- Fan or Stirrer -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - spray']" mode="intelem">
		<!-- Spraying Device -->
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.2;0);(0.2;-0.2)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>2037</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - spray']" mode="intelemname">
		<!-- Spraying Device -->
		<xsl:text>Spray</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - spray']" mode="extifname">
		<!-- Spraying Device -->
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which -->
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - pneum']" mode="intelem">
		<!-- Pneumatic Line -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pneum']" mode="intelemname">
		<!-- Pneumatic Line -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pneum']" mode="extifname">
		<!-- Pneumatic Line -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - pneumv']" mode="intelem">
		<!-- Pneumatic Line, vertical -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pneumv']" mode="intelemname">
		<!-- Pneumatic Line, vertical -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pneumv']" mode="extifname">
		<!-- Pneumatic Line, vertical -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - measure']" mode="intelem">
		<!-- Measurement -->
		<InternalElement>
			<xsl:message terminate="yes">
				<xsl:text>error in diagram: do not use measure from ChemEng but from IEC62424</xsl:text>
			</xsl:message>
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - measure']" mode="intelemname">
		<!-- Measurement -->
		<xsl:message terminate="yes">
			<xsl:text>error in diagram: do not use measure from ChemEng but from IEC62424</xsl:text>
		</xsl:message>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - measure']" mode="extifname">
		<!-- Measurement -->
		<xsl:param name="connection" />

		<xsl:message terminate="yes">
			<xsl:text>error in diagram: do not use measure from ChemEng but from IEC62424</xsl:text>
		</xsl:message>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - hx' or @type='chemeng - hxv']" mode="intelem">
		<!-- Simple Heat Exchanger -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>stth_SUC@PPE_SystemUnitLib2/TubeShellHeatExchanger</xsl:text>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/HeatExchangerRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - hx' or @type='chemeng - hxv']" mode="intelemname">
		<!-- Simple Heat Exchanger -->
		<xsl:text>HE</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - hx']" mode="extifname">
		<!-- Simple Heat Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which -->
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>PTempIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>PTempOut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - hxv']" mode="extifname">
		<!-- Simple Heat Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which -->
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>PTempIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>PTempOut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- Water Cooler, horizontal or vertical -->
	<xsl:template match="dia:object[@type='chemeng - wcool' or @type='chemeng - wcoolv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<!-- <Attribute Name="NominalVolume" Unit="m^3" AttributeDataType="xs:double"> <Value>0.0</Value> </Attribute> -->
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(1,0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8017</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="CoolingIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="CoolingOut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="stth_PPE@stth_PPE_Lib/WaterCoolerReqest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - wcool' or @type='chemeng - wcoolv']" mode="intelemname">
		<xsl:text>HE</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - wcool']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which if its done another way than wcoolv, we got a problem above -->
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>CoolingIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>CoolingOut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - wcoolv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which -->
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>CoolingIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>CoolingOut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - SaT-fixedtube']" mode="intelem">
		<!-- Fixed-Sheet Exchanger -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - SaT-fixedtube']" mode="intelemname">
		<!-- Fixed-Sheet Exchanger -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - SaT-fixedtube']" mode="extifname">
		<!-- Fixed-Sheet Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - SaT-floatinghead']" mode="intelem">
		<!-- Floating-Head or U-Tube Exchanger -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - SaT-floatinghead']" mode="intelemname">
		<!-- Floating-Head or U-Tube Exchanger -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - SaT-floatinghead']" mode="extifname">
		<!-- Floating-Head or U-Tube Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - kettle']" mode="intelem">
		<!-- Kettle Reboiler -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - kettle']" mode="intelemname">
		<!-- Kettle Reboiler -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - kettle']" mode="extifname">
		<!-- Kettle Reboiler -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - aircooler']" mode="intelem">
		<!-- Air Cooler -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - aircooler']" mode="intelemname">
		<!-- Air Cooler -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - aircooler']" mode="extifname">
		<!-- Air Cooler -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - airforced']" mode="intelem">
		<!-- Forced-Flow Air Cooler -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - airforced']" mode="intelemname">
		<!-- Forced-Flow Air Cooler -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - airforced']" mode="extifname">
		<!-- Forced-Flow Air Cooler -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - airinduced']" mode="intelem">
		<!-- Induced-Flow Air Cooler -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - airinduced']" mode="intelemname">
		<!-- Induced-Flow Air Cooler -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - airinduced']" mode="extifname">
		<!-- Induced-Flow Air Cooler -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - plate']" mode="intelem">
		<!-- Plate Exchanger -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - plate']" mode="intelemname">
		<!-- Plate Exchanger -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - plate']" mode="extifname">
		<!-- Plate Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - doublepipe']" mode="intelem">
		<!-- Double-Pipe Exchanger -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - doublepipe']" mode="intelemname">
		<!-- Double-Pipe Exchanger -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - doublepipe']" mode="extifname">
		<!-- Double-Pipe Exchanger -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - coil']" mode="intelem">
		<!-- Heating/Cooling Coil -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - coil']" mode="intelemname">
		<!-- Heating/Cooling Coil -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - coil']" mode="extifname">
		<!-- Heating/Cooling Coil -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - coilv']" mode="intelem">
		<!-- Heating/Cooling Coil, vertical -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - coilv']" mode="intelemname">
		<!-- Heating/Cooling Coil, vertical -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - coilv']" mode="extifname">
		<!-- Heating/Cooling Coil, vertical -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - furnace']" mode="intelem">
		<!-- Simple Furnace -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - furnace']" mode="intelemname">
		<!-- Simple Furnace -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - furnace']" mode="extifname">
		<!-- Simple Furnace -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - vessel']" mode="intelem">
		<!-- Simple Vessel -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>stth_SUC@PPE_SystemUnitLib2/LVessel</xsl:text>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<ExternalInterface Name="PIn" /> <!-- RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint"> <Attribute Name="DN"> <Value>32</Value> </Attribute> <Attribute Name="Position"> <Value>S01P2:0.5</Value> </Attribute> </ExternalInterface> -->
			<ExternalInterface Name="POut" /> <!-- RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint"> <Attribute Name="DN"> <Value>32</Value> </Attribute> <Attribute Name="Position"> <Value>S01P2:-0.5</Value> </Attribute> </ExternalInterface> -->
			<ExternalInterface Name="PInL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0.3</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="PInR" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0.3</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="POutL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:-0.3</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="POutR" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:-0.3</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/LVesselRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - vessel']" mode="intelemname">
		<!-- Simple Vessel -->
		<xsl:text>Vessel</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - vessel']" mode="extifname">
		<!-- Simple Vessel -->
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TO-DO which is which -->
			<xsl:when test="$connection='14'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='15'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='8'">
				<xsl:text>PInR</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='12'">
				<xsl:text>POutR</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>PInL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>POutL</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - knockout']" mode="intelem">
		<!-- Knock-out Drum (with demister pad) -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - knockout']" mode="intelemname">
		<!-- Knock-out Drum (with demister pad) -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - knockout']" mode="extifname">
		<!-- Knock-out Drum (with demister pad) -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - traycol']" mode="intelem">
		<!-- Tray Column, simple -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - traycol']" mode="intelemname">
		<!-- Tray Column, simple -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - traycol']" mode="extifname">
		<!-- Tray Column, simple -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - bigtraycol']" mode="intelem">
		<!-- Tray Column, detailed -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - bigtraycol']" mode="intelemname">
		<!-- Tray Column, detailed -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - bigtraycol']" mode="extifname">
		<!-- Tray Column, detailed -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - fluidcont']" mode="intelem">
		<!-- Fluid Contacting Vessel, simple -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - fluidcont']" mode="intelemname">
		<!-- Fluid Contacting Vessel, simple -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - fluidcont']" mode="extifname">
		<!-- Fluid Contacting Vessel, simple -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - reactor']" mode="intelem">
		<!-- Reactor or Absorption Vessel, simple -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - reactor']" mode="intelemname">
		<!-- Reactor or Absorption Vessel, simple -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - reactor']" mode="extifname">
		<!-- Reactor or Absorption Vessel, simple -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - autoclave']" mode="intelem">
		<!-- Autoclave -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - autoclave']" mode="intelemname">
		<!-- Autoclave -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - autoclave']" mode="extifname">
		<!-- Autoclave -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - tank']" mode="intelem">
		<!-- Open Tank -->
		<InternalElement>
			<!--<xsl:attribute name="RefBaseSystemUnitPath"> <xsl:text>TO-DO</xsl:text> </xsl:attribute> -->
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F2:(0;0);(1;1)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>2061</Value>
			</Attribute>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0.3</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:-0.3</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - tank']" mode="intelemname">
		<!-- Open Tank -->
		<xsl:text>Tank</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - tank']" mode="extifname">
		<!-- Open Tank -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='10'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - settling']" mode="intelem">
		<!-- Clarifier or Settling Tank -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - settling']" mode="intelemname">
		<!-- Clarifier or Settling Tank -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - settling']" mode="extifname">
		<!-- Clarifier or Settling Tank -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - sealedtank']" mode="intelem">
		<!-- Sealed Tank -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - sealedtank']" mode="intelemname">
		<!-- Sealed Tank -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - sealedtank']" mode="extifname">
		<!-- Sealed Tank -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - coveredtank']" mode="intelem">
		<!-- Covered tank -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - coveredtank']" mode="intelemname">
		<!-- Covered tank -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - coveredtank']" mode="extifname">
		<!-- Covered tank -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - fixedroof']" mode="intelem">
		<!-- Tank with Fixed Roof -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - fixedroof']" mode="intelemname">
		<!-- Tank with Fixed Roof -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - fixedroof']" mode="extifname">
		<!-- Tank with Fixed Roof -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - floatingroof']" mode="intelem">
		<!-- Tank with Floating Roof -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - floatingroof']" mode="intelemname">
		<!-- Tank with Floating Roof -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - floatingroof']" mode="extifname">
		<!-- Tank with Floating Roof -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - storagesphere']" mode="intelem">
		<!-- Storage Sphere -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - storagesphere']" mode="intelemname">
		<!-- Storage Sphere -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - storagesphere']" mode="extifname">
		<!-- Storage Sphere -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - gasholder']" mode="intelem">
		<!-- Gas Holder, basic -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - gasholder']" mode="intelemname">
		<!-- Gas Holder, basic -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - gasholder']" mode="extifname">
		<!-- Gas Holder, basic -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - pump']" mode="intelem">
		<!-- Centrifugal Pump or Fan -->
		<InternalElement>
			<!-- <xsl:attribute name="RefBaseSystemUnitPath"> <xsl:text>TO-DO</xsl:text> </xsl:attribute> -->
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<!-- <Attribute Name="NominalVolume" Unit="m^3" AttributeDataType="xs:double"> <Value>0.0</Value> </Attribute> -->
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(1,0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>2322</Value>
			</Attribute>
			<Attribute Name="MaximalFlowrate" Unit="m^3/s">
				<Value>0.01</Value>
			</Attribute>
			<!-- <Attribute Name="MaximalDeliveryHeight">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="NominalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<!- - TODO Pandix Pump Characteristic - ->
			<Attribute Name="PumpCharacteristic" AttributeDataType="xs:string" Unit="-">
				<Value>TODO</Value>
			</Attribute> -->
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PumpRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pump']" mode="intelemname">
		<!-- Centrifugal Pump or Fan -->
		<xsl:text>Pump</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - pump']" mode="extifname">
		<!-- Centrifugal Pump or Fan -->
		<xsl:param name="connection" />
		<xsl:choose>
			<!-- TODO symbol has no direction where is in, where is out? -->
			<xsl:when test="$connection='3'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='0'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - displa']" mode="intelem">
		<!-- Positive Displacement Rotary Pump or Compressor -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - displa']" mode="intelemname">
		<!-- Positive Displacement Rotary Pump or Compressor -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - displa']" mode="extifname">
		<!-- Positive Displacement Rotary Pump or Compressor -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - recipr']" mode="intelem">
		<!-- Reciprocating Compressor or Pump -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - recipr']" mode="intelemname">
		<!-- Reciprocating Compressor or Pump -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - recipr']" mode="extifname">
		<!-- Reciprocating Compressor or Pump -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - flowfan']" mode="intelem">
		<!-- Axial Flow Fan -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - flowfan']" mode="intelemname">
		<!-- Axial Flow Fan -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - flowfan']" mode="extifname">
		<!-- Axial Flow Fan -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - compr']" mode="intelem">
		<!-- Compressor or Turbine -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - compr']" mode="intelemname">
		<!-- Compressor or Turbine -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - compr']" mode="extifname">
		<!-- Compressor or Turbine -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - ejector']" mode="intelem">
		<!-- Ejector or Injector -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - ejector']" mode="intelemname">
		<!-- Ejector or Injector -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - ejector']" mode="extifname">
		<!-- Ejector or Injector -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - ACgen']" mode="intelem">
		<!-- AC Generator -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - ACgen']" mode="intelemname">
		<!-- AC Generator -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - ACgen']" mode="extifname">
		<!-- AC Generator -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - val']" mode="intelem">
		<!-- Valve -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - val']" mode="intelemname">
		<!-- Valve -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - val']" mode="extifname">
		<!-- Valve -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - valv']" mode="intelem">
		<!-- Valve, vertical -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - valv']" mode="intelemname">
		<!-- Valve, vertical -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - valv']" mode="extifname">
		<!-- Valve, vertical -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - regval']" mode="intelem">
		<!-- Regulable Valve -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - regval']" mode="intelemname">
		<!-- Regulable Valve -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - regval']" mode="extifname">
		<!-- Regulable Valve -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemeng - regvalv']" mode="intelem">
		<!-- Regulable Valve, vertical -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - regvalv']" mode="intelemname">
		<!-- Regulable Valve, vertical -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemeng - regvalv']" mode="extifname">
		<!-- Regulable Valve, vertical -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - filter']" mode="intelem">
		<!-- Basic Filter -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - filter']" mode="intelemname">
		<!-- Basic Filter -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - filter']" mode="extifname">
		<!-- Basic Filter -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - mixer']" mode="intelem">
		<!-- Mixer -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - mixer']" mode="intelemname">
		<!-- Mixer -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - mixer']" mode="extifname">
		<!-- Mixer -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - cyclone']" mode="intelem">
		<!-- Cyclone and hydrocyclone -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - cyclone']" mode="intelemname">
		<!-- Cyclone and hydrocyclone -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - cyclone']" mode="extifname">
		<!-- Cyclone and hydrocyclone -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - spraydrier']" mode="intelem">
		<!-- Spray Drier -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - spraydrier']" mode="intelemname">
		<!-- Spray Drier -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - spraydrier']" mode="extifname">
		<!-- Spray Drier -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="dia:object[@type='chemengtmp - centrifuge']" mode="intelem">
		<!-- Centrifuge -->
		<InternalElement>
			<xsl:attribute name="RefBaseSystemUnitPath">
				<xsl:text>TO-DO</xsl:text>
				<xsl:message terminate="yes">
					<xsl:text>not implemented yet: </xsl:text>
					<xsl:value-of select="@type" />
				</xsl:message>
			</xsl:attribute>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<RoleRequirements RefBaseRoleClassPath="TO-DO" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - centrifuge']" mode="intelemname">
		<!-- Centrifuge -->
		<xsl:text>TO-DO</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='chemengtmp - centrifuge']" mode="extifname">
		<!-- Centrifuge -->
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='-1'">
				<xsl:text>TO-DO</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>
