<?xml version="1.0" encoding="UTF-8"?>
<!--
COPYRIGHT

Dia2CAEX - a diagram to CAEX converter
Copyright (C) 2012,2014 Stefan Theurich, Prozesskommunikation, TU Dresden

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<!--
Any changes should be verified on a caex checker like http://caex.inf.tu-dresden.de
-->

<xsl:stylesheet
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		
		xmlns:fn="http://www.w3.org/2005/xpath-functions"
		
		xmlns:dia="http://www.lysator.liu.se/~alla/dia/"
		
		xmlns:uuid="java:java.util.UUID"
		
		exclude-result-prefixes="xsl xsi fn uuid dia"
		
		version="2.0">

	<!-- every dia figure needs two or three templates:
		
		mandatory: <xsl:template match="dia:object[@type='???']" mode="intelem" />
		prints out the <InternalElement /> tag that declares the object
		
		optional: 
		<xsl:template match="dia:object[@type='???']" mode="intelemname" />
		prints out the plain name of th element if there is a textbox in the shape,
		this text (without any line breaks and spaces) will be used 
		as name and this template will only be called, if there is no string given
		
		mandatory <xsl:template match="dia:object[@type='???']" mode="extifname" /> <xsl:param name="connection" />
		prints the name of  the external interface of the object
		corresponding to the given connector number (as string)
	-->

	<!-- a group name -->
	<xsl:template match="dia:object[@type='IEC62424 - internal_element_name']" mode="intelem" />
	<xsl:template match="dia:object[@type='IEC62424 - internal_element_name']" mode="intelemname">
		<!-- fallback for text -->
		<xsl:text>PlantSection</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - internal_element_name']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:message terminate="yes">
			<xsl:text>there is no external interface in internal_element_name at diagram position </xsl:text>
			<xsl:apply-templates select="." mode="objectpos" />
		</xsl:message>
	</xsl:template>

	<!-- two connected pipe ends -->
	<xsl:template match="dia:object[@type='IEC62424 - pipe_connection' or @type='IEC62424 - pipe_connectionv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P2:0</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>511</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_connection' or @type='IEC62424 - pipe_connectionv']" mode="intelemname">
		<xsl:text>PipeDNAdapter</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_connection' or @type='IEC62424 - pipe_connectionv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- an closed pipe end -->
	<xsl:template match="dia:object[@type='IEC62424 - blank_flange' or @type='IEC62424 - blank_flangev']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8151</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/BlankFlangeRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - blank_flange' or @type='IEC62424 - blank_flangev']" mode="intelemname">
		<xsl:text>BlankFlange</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - blank_flange' or @type='IEC62424 - blank_flangev']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- an open pipe end -->
	<xsl:template match="dia:object[@type='IEC62424 - open_flange' or @type='IEC62424 - open_flangev']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8126</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="PEn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductEnvironmentPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/OpenFlangeRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - open_flange' or @type='IEC62424 - open_flangev']" mode="intelemname">
		<xsl:text>OpenFlange</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - open_flange' or @type='IEC62424 - open_flangev']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<!-- an adapter to change the pipe diameter -->
	<xsl:template match="dia:object[@type='IEC62424 - pipe_dn_adapter' or @type='IEC62424 - pipe_dn_adapterv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>516</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>40</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_dn_adapter' or @type='IEC62424 - pipe_dn_adapterv']" mode="intelemname">
		<xsl:text>PipeDNAdapter</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_dn_adapter' or @type='IEC62424 - pipe_dn_adapterv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a pipe junction with three nozzles -->
	<xsl:template match="dia:object[@type='IEC62424 - junction3' or @type='IEC62424 - junction3v']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>8155</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P3" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PipeJunctionRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction3' or @type='IEC62424 - junction3v']" mode="intelemname">
		<xsl:text>PipeJunction</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction3' or @type='IEC62424 - junction3v']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>P3</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a pipe junction with four nozzles -->
	<xsl:template match="dia:object[@type='IEC62424 - junction4']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>8155</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P3" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P4" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/Pipe4JunctionRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction4']" mode="intelemname">
		<xsl:text>PipeJunction</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction4']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>P3</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>P4</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a pipe junction with five nozzles -->
	<xsl:template match="dia:object[@type='IEC62424 - junction5' or @type='IEC62424 - junction5v']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>8155</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P3" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P4" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'4'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P5" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/Pipe5JunctionRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction5' or @type='IEC62424 - junction5v']" mode="intelemname">
		<xsl:text>PipeJunction</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - junction5' or @type='IEC62424 - junction5v']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>P3</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>P4</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>P5</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- input and input v -->
	<xsl:template match="dia:object[@type='IEC62424 - input' or @type='IEC62424 - inputv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>C0103</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="ExternPIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - input' or @type='IEC62424 - inputv']" mode="intelemname">
		<!-- fallback for text -->
		<xsl:text>Input</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - input' or @type='IEC62424 - inputv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- output and output v -->
	<xsl:template match="dia:object[@type='IEC62424 - output' or @type='IEC62424 - outputv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>C0103</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="ExternPOut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - output' or @type='IEC62424 - outputv']" mode="intelemname">
		<!-- fallback for text -->
		<xsl:text>Output</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - output' or @type='IEC62424 - outputv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- inoutputv -->
	<xsl:template match="dia:object[@type='IEC62424 - inoutputv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>C0103</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PInOut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="ExternPInOut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - inoutputv']" mode="intelemname">
		<!-- fallback for text -->
		<xsl:text>InOutput</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - inoutputv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PInOut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- measurement request -->
	<xsl:template match="dia:object[@type='IEC62424 - measurement']" mode="intelem">
		<InternalElement>
			<xsl:variable name="text1" select="substring-before(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:variable name="text2" select="substring-after(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="RemoteType">
				<Value>Central</Value>
			</Attribute>
			<Attribute Name="SignalCode">
				<Value>
					<xsl:value-of select="substring($text1,2,1)" />
				</Value>
			</Attribute>
			<Attribute Name="Category">
				<Value>
					<xsl:value-of select="substring($text1,1,1)" />
				</Value>
			</Attribute>
			<Attribute Name="ReferenceCode">
				<Value>A</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SensorProcessInterface" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="VAL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="L" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="H" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'4'" />
			</xsl:apply-templates>
			<ExternalInterface Name="LL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'5'" />
			</xsl:apply-templates>
			<ExternalInterface Name="HH" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PCE@PandIX_PCE/SensorRequest" />
		</InternalElement>
	</xsl:template>
	<!-- <xsl:template match="dia:object[@type='IEC62424 - measurement']" mode="intelemname"> -->
	<xsl:template match="dia:object[@type='IEC62424 - measurement']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>VAL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>L</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>H</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>LL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>HH</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- measurement diff request -->
	<xsl:template match="dia:object[@type='IEC62424 - measurement_diff']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<xsl:variable name="text1" select="substring-before(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:variable name="text2" select="substring-after(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<Attribute Name="RemoteType">
				<Value>Central</Value>
			</Attribute>
			<Attribute Name="SignalCode">
				<Value>
					<xsl:value-of select="substring($text1,3,1)" />
				</Value>
			</Attribute>
			<Attribute Name="Category">
				<Value>
					<xsl:value-of select="substring($text1,1,1)" />
				</Value>
			</Attribute>
			<Attribute Name="ReferenceCode">
				<Value>D</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SensorProcessInterface" />
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SensorProcessInterface" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="VAL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="L" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'4'" />
			</xsl:apply-templates>
			<ExternalInterface Name="H" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'5'" />
			</xsl:apply-templates>
			<ExternalInterface Name="LL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'6'" />
			</xsl:apply-templates>
			<ExternalInterface Name="HH" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PCE@PandIX_PCE/SensorRequest" />
		</InternalElement>
	</xsl:template>
	<!-- <xsl:template match="dia:object[@type='IEC62424 - measurement_diff']" mode="intelemname"> -->
	<xsl:template match="dia:object[@type='IEC62424 - measurement_diff']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>P2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>VAL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>L</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>H</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>LL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>HH</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- actuator request -->
	<xsl:template match="dia:object[@type='IEC62424 - actuator']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<xsl:variable name="text1" select="substring-before(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:variable name="text2" select="substring-after(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<Attribute Name="RemoteType">
				<Value>Central</Value>
			</Attribute>
			<Attribute Name="SignalCode">
				<Value>I</Value>
			</Attribute>
			<Attribute Name="Category">
				<Value>
					<xsl:value-of select="substring($text1,1,1)" />
				</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuatorProcessInterface" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="L" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="H" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'4'" />
			</xsl:apply-templates>
			<ExternalInterface Name="LL" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'5'" />
			</xsl:apply-templates>
			<ExternalInterface Name="HH" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<ExternalInterface Name="LOCK" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<ExternalInterface Name="NLOCK" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PCE@PandIX_PCE/ActuatorRequest" />
		</InternalElement>
	</xsl:template>
	<!-- <xsl:template match="dia:object[@type='IEC62424 - actuator']" mode="intelemname"> -->
	<xsl:template match="dia:object[@type='IEC62424 - actuator']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>W</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>L</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>H</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>LL</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>HH</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- control function reqest -->
	<xsl:template match="dia:object[@type='IEC62424 - control']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<xsl:variable name="text1" select="substring-before(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:variable name="text2" select="substring-after(replace(dia:attribute[@name='text']/dia:composite/dia:attribute[@name='string']/dia:string,'#', ''),'&#xA;')" />
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<Attribute Name="RemoteType">
				<Value>Central</Value>
			</Attribute>
			<Attribute Name="SignalCode">
				<Value>
					<xsl:value-of select="substring($text1,2,1)" />
				</Value>
			</Attribute>
			<Attribute Name="Category">
				<Value>U</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="Y" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSource" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W3" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'4'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W4" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'5'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W5" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'6'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W6" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'7'" />
			</xsl:apply-templates>
			<ExternalInterface Name="W7" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<ExternalInterface Name="LOCK" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<ExternalInterface Name="NLOCK" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<ExternalInterface Name="X" RefBaseClassPath="PandIX_ICL@PandIX_ICL/SignalSink" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PCE@PandIX_PCE/ControlFunctionRequest" />
		</InternalElement>
	</xsl:template>
	<!-- <xsl:template match="dia:object[@type='IEC62424 - control']" mode="intelemname"> -->
	<xsl:template match="dia:object[@type='IEC62424 - control']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>Y</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>W1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>W2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>W3</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>W4</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>W5</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>W6</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='7'">
				<xsl:text>W7</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a common valve -->
	<xsl:template match="dia:object[@type='IEC62424 - common_valve' or @type='IEC62424 - common_valvev']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2101</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<Attribute Name="Kvs">
				<Value>3</Value>
			</Attribute>
			<Attribute Name="SafeState">
				<Value>none</Value>
			</Attribute>
			<!-- TODO Pandix Valve Characteristic -->
			<Attribute Name="ValveCharacteristic" AttributeDataType="xs:string" Unit="-">
				<Value>linear</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="Y" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/ValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_valve' or @type='IEC62424 - common_valvev']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_valve' or @type='IEC62424 - common_valvev']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>Y</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a three-way valve -->
	<xsl:template match="dia:object[@type='IEC62424 - common_valve3' or @type='IEC62424 - common_valve3v']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2103</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<Attribute Name="SafeState">
				<Value>none</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2-1" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="P2-2" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="Y" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/ThreeWayValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_valve3' or @type='IEC62424 - common_valve3v']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_valve3' or @type='IEC62424 - common_valve3v']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='2'">
				<xsl:text>P1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='0'">
				<xsl:text>P2-1</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>P2-2</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>Y</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a check valve, no flow in opposite direction (POut->PIn) -->
	<xsl:template match="dia:object[@type='IEC62424 - check_valve' or @type='IEC62424 - check_valvev']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>X2113</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/CheckValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - check_valve' or @type='IEC62424 - check_valvev']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - check_valve' or @type='IEC62424 - check_valvev']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a safety valve, opens at a specific pressure an closes below -->
	<xsl:template match="dia:object[@type='IEC62424 - safety_valve' or @type='IEC62424 - safety_valvev']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2112</Value>
			</Attribute>
			<Attribute Name="PressureLimit">
				<Value>10</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/SafetyValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - safety_valve' or @type='IEC62424 - safety_valvev']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - safety_valve' or @type='IEC62424 - safety_valvev']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a spring valve, opens at a specific pressure an closes below -->
	<xsl:template match="dia:object[@type='IEC62424 - spring_valve']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>X2125</Value>
			</Attribute>
			<Attribute Name="PressureLimit">
				<Value>5</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/SafetyValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - spring_valve']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - spring_valve']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a rupture disk, opens at a specific pressure and doesn't close again -->
	<xsl:template match="dia:object[@type='IEC62424 - rupture_disk' or @type='IEC62424 - rupture_diskv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8080</Value>
			</Attribute>
			<Attribute Name="PressureLimit">
				<Value>15</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/RuptureDiskRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - rupture_disk' or @type='IEC62424 - rupture_diskv']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - rupture_disk' or @type='IEC62424 - rupture_diskv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a manual valve -->
	<xsl:template match="dia:object[@type='IEC62424 - stop_cock' or @type='IEC62424 - stop_cockv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8071</Value>
			</Attribute>
			<Attribute Name="SafeState">
				<Value>none</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/ManualValveRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - stop_cock' or @type='IEC62424 - stop_cockv']" mode="intelemname">
		<xsl:text>V</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - stop_cock' or @type='IEC62424 - stop_cockv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a common fluid pump -->
	<xsl:template match="dia:object[@type='IEC62424 - common_pump' or @type='IEC62424 - common_pumpv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2301</Value>
			</Attribute>
			<Attribute Name="MaximalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.01</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<!-- <Attribute Name="MaximalDeliveryHeight">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="NominalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<!- - TODO Pandix Pump Characteristic - ->
			<Attribute Name="PumpCharacteristic" AttributeDataType="xs:string" Unit="-">
				<Value>TODO</Value>
			</Attribute> -->
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PumpRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_pump' or @type='IEC62424 - common_pumpv']" mode="intelemname">
		<xsl:text>P</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_pump' or @type='IEC62424 - common_pumpv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a rotary pump -->
	<xsl:template match="dia:object[@type='IEC62424 - rotary_pump' or @type='IEC62424 - rotary_pumpv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2322</Value>
			</Attribute>
			<Attribute Name="MaximalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.01</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<Attribute Name="MaximalDeliveryHeight">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="NominalDeliveryHeight">
				<Value>15</Value>
			</Attribute>
			<Attribute Name="NominalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<!-- TODO Pandix Pump Characteristic -->
			<Attribute Name="PumpCharacteristic" AttributeDataType="xs:string" Unit="-">
				<Value>TODO</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/RPumpRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - rotary_pump' or @type='IEC62424 - rotary_pumpv']" mode="intelemname">
		<xsl:text>P</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - rotary_pump' or @type='IEC62424 - rotary_pumpv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!--a common displacement pump -->
	<xsl:template match="dia:object[@type='IEC62424 - displacement_pump' or @type='IEC62424 - displacement_pumpv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8135</Value>
			</Attribute>
			<Attribute Name="MaximalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.01</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<Attribute Name="MaximalBackpressure">
				<Value>5</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/DPumpRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - displacement_pump' or @type='IEC62424 - displacement_pumpv']" mode="intelemname">
		<xsl:text>P</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - displacement_pump' or @type='IEC62424 - displacement_pumpv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a common gas pump / compressor -->
	<xsl:template match="dia:object[@type='IEC62424 - common_gas_pump' or @type='IEC62424 - common_gas_pumpv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2302</Value>
			</Attribute>
			<Attribute Name="MaximalFlowrate" Unit="m^3/s">
				<Value>0.2</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.01</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.1;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/CompressorRequest" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_gas_pump' or @type='IEC62424 - common_gas_pumpv']" mode="intelemname">
		<xsl:text>P</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - common_gas_pump' or @type='IEC62424 - common_gas_pumpv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a heater -->
	<xsl:template match="dia:object[@type='IEC62424 - pipe_heater' or @type='IEC62424 - pipe_heaterv']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>C2041</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F1:(0.5;0)</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'2'" />
			</xsl:apply-templates>
			<ExternalInterface Name="N" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_heater' or @type='IEC62424 - pipe_heaterv']" mode="intelemname">
		<xsl:text>W</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - pipe_heater' or @type='IEC62424 - pipe_heaterv']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>N</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a fluid contacting vessel with two connected contact fillings and a spray input -->
	<xsl:template match="dia:object[@type='IEC62424 - vessel_with_level']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="ISO10628Type">
				<Value>2062</Value>
			</Attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F2:(0;0);(0.1;0.07);(0.5;0.5);(0.9;0.93);(1;1)</Value>
			</Attribute>
			<ExternalInterface Name="TopCenter" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="Bottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="Level" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="TopLeft" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.3</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="TopRight" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.3</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LeftTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LeftBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/VesselRequest" />
			<MappingObject>
				<InterfaceNameMapping SystemUnitInterfaceName="TopCenter" RoleInterfaceName="PIn" />
				<InterfaceNameMapping SystemUnitInterfaceName="Bottom" RoleInterfaceName="POut" />
			</MappingObject>
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - vessel_with_level']" mode="intelemname">
		<xsl:text>B</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - vessel_with_level']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>TopCenter</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>Bottom</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>Level</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>TopLeft</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>TopRight</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>LeftTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>RightTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='7'">
				<xsl:text>LeftBottom</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='8'">
				<xsl:text>RightBottom</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a fluid contacting vessel with two connected contact fillings and a spray input -->
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F2:(0;0);(0.1;0.07);(0.5;0.5);(0.9;0.93);(1;1)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8015</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="SprayIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="Top" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="Bottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="LeftTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="UpperBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.25</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="MiddleBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LowerBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.25</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LeftBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2']" mode="intelemname">
		<xsl:text>K</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>SprayIn</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='1'">
				<xsl:text>Top</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>Bottom</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='3'">
				<xsl:text>LeftTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>UpperBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>MiddleBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>LowerBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='7'">
				<xsl:text>LeftBottom</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='8'">
				<xsl:text>RightBottom</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a fluid contacting vessel with two separate contact fillings and two spray inputs -->
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2in']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F2:(0;0);(0.1;0.07);(0.5;0.5);(0.9;0.93);(1;1)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8016</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="SprayInTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.4</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="SprayInMiddle" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="Top" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="Bottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="LeftTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="UpperBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.25</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="MiddleBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LowerBed" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.25</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LeftBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2in']" mode="intelemname">
		<xsl:text>K</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - fluid_contacting_vessel2in']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>SprayInTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>SprayInMiddle</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='2'">
				<xsl:text>Top</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='3'">
				<xsl:text>Bottom</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='4'">
				<xsl:text>LeftTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='5'">
				<xsl:text>UpperBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>MiddleBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='7'">
				<xsl:text>LowerBed</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='8'">
				<xsl:text>LeftBottom</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='9'">
				<xsl:text>RightBottom</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<!-- a gas heater using liquid as contact medium -->
	<xsl:template match="dia:object[@type='IEC62424 - gas_heater_fluid_contact']" mode="intelem">
		<InternalElement>
			<xsl:attribute name="Name">
				<xsl:apply-templates select="." mode="intelemname" />
			</xsl:attribute>
			<xsl:attribute name="ID">
				<xsl:value-of select="uuid:randomUUID()" />
			</xsl:attribute>
			<Attribute Name="PN">
				<Value>20</Value>
			</Attribute>
			<Attribute Name="TotalVolume">
				<Value>0.0</Value>
			</Attribute>
			<Attribute Name="Position">
				<Value>S01P0:</Value>
			</Attribute>
			<Attribute Name="Form">
				<Value>S01F2:(0;0);(0.1;0.07);(0.5;0.5);(0.9;0.93);(1;1)</Value>
			</Attribute>
			<Attribute Name="ISO10628Type">
				<Value>X8100</Value>
			</Attribute>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'0'" />
			</xsl:apply-templates>
			<ExternalInterface Name="PIn" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.4</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_outgoing_connection">
				<xsl:with-param name="connection" select="'1'" />
			</xsl:apply-templates>
			<ExternalInterface Name="POut" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="Bottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.5</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<xsl:apply-templates select="." mode="require_connected_connector">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<xsl:apply-templates select="." mode="require_incoming_connection">
				<xsl:with-param name="connection" select="'3'" />
			</xsl:apply-templates>
			<ExternalInterface Name="Heater" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ActuationInputPoint" />
			<ExternalInterface Name="Level" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>

			<ExternalInterface Name="LeftTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightTop" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="LeftMiddle" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:0</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<ExternalInterface Name="RightBottom" RefBaseClassPath="PandIX_ICL@PandIX_ICL/ProductConnectionPoint">
				<Attribute Name="Position">
					<Value>S01P2:-0.45</Value>
				</Attribute>
				<Attribute Name="DN">
					<Value>32</Value>
				</Attribute>
			</ExternalInterface>
			<RoleRequirements RefBaseRoleClassPath="PandIX_PPE@PPE_BasicLib/PPE-Request" />
		</InternalElement>
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - gas_heater_fluid_contact']" mode="intelemname">
		<xsl:text>K</xsl:text>
		<xsl:value-of select="@id" />
	</xsl:template>
	<xsl:template match="dia:object[@type='IEC62424 - gas_heater_fluid_contact']" mode="extifname">
		<xsl:param name="connection" />
		<xsl:choose>
			<xsl:when test="$connection='0'">
				<xsl:text>PIn</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='1'">
				<xsl:text>POut</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='2'">
				<xsl:text>Bottom</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='3'">
				<xsl:text>Heater</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='4'">
				<xsl:text>Level</xsl:text>
			</xsl:when>

			<xsl:when test="$connection='5'">
				<xsl:text>LeftTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='6'">
				<xsl:text>RightTop</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='7'">
				<xsl:text>LeftMiddle</xsl:text>
			</xsl:when>
			<xsl:when test="$connection='8'">
				<xsl:text>RightBottom</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:apply-templates select="." mode="unsupported_connector">
					<xsl:with-param name="connection" select="$connection" />
				</xsl:apply-templates>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>